; =============================================================================
; Filename    : DrawOrderByLayer.lsp
; Date        : 27.06.06
; Author      : jme
; Copyright   : MENZI ENGINEERING GmbH, Switzerland
; Revision  1 : __.__.__ ___ -
; -----------------------------------------------------------------------------
; Known bugs:
; - None
; -----------------------------------------------------------------------------
; Description:
; Set objects draworder by layer name and saves the settings in the drawing.
; -----------------------------------------------------------------------------
; Global variables:
; Me:AcD  Me:Dcl  Me:Fst  Me:Lst  Me:Pos
; -----------------------------------------------------------------------------
; Internal LISP-functions:
; MeAll2String  MeDoSpin  MeEditList  MeGetDictRecs  MeGetLayerNames
; MeGetMassoc  MeList2Array  MeList2String  MeLoadDcl  MeMoveControl
; MeMoveListItems  MeOpenDcl  MeSetDictRecs  MeString2List
; -----------------------------------------------------------------------------
; External LISP-functions:
; 
; -----------------------------------------------------------------------------
; Version notes:
; AutoCAD:	Version:	Language:	AddIns:
; 16.0+		1.00		English		...
; -----------------------------------------------------------------------------
;
; == Message on loading =======================================================
;
(princ "\nDrawOrderByLayer v1.00")
;
; == Main =====================================================================
;
(defun C:DrawOrderByLayer ( / AddLst CurLst CurSet Dcl_Id DefLst LayLst LstCnt
                              OldCmd SelLst)
 (cond
  ((< (atof (getvar "ACADVER")) 16.0)
   (alert "DrawOrderByLayer requires AutoCAD 2004 or higher. ")
  )
  ((and
    (not (null (vl-load-com)))
    (or Me:AcD (setq Me:Acd (vla-get-ActiveDocument (vlax-get-acad-object))))
    (< (length (setq LayLst (MeGetLayerNames))) 2)
   )
   (alert "One layer makes no sense to sort. ")
  )
  ((not (setq Dcl_Id (MeLoadDcl "DrawOrderByLayer"))))
  (T
   (vla-StartUndoMark Me:AcD)
   (setq LayLst (acad_strlsort LayLst)
         DefLst (MeGetDictRecs "DrawOrderByLayer" "DefaultLayerList")
         DefLst (MeGetMassoc 1 DefLst)
         Me:Prp nil
   )
   (MeOpenDcl "dol_main" Me:Pos Dcl_Id)
   (cond
    ((> (length DefLst) (length LayLst))
     (foreach memb DefLst
      (if (member memb LayLst)
       (setq CurLst (cons memb CurLst))
      )
     )
     (setq CurLst (reverse CurLst))
    )
    ((< (length DefLst) (length LayLst))
     (foreach memb LayLst
      (if (member memb DefLst)
       (setq CurLst (cons memb CurLst))
       (setq AddLst (cons memb AddLst))
      )
     )
     (setq CurLst (reverse CurLst)
           AddLst (reverse AddLst)
     )
     (if (and CurLst AddLst) (mode_tile "lay_up" 0))
    )
    ((setq CurLst DefLst))
   )
   (setq LstCnt (length CurLst)
         Me:Lst (append CurLst AddLst)
   )
   (start_list "laylst")
   (mapcar 'add_list Me:Lst)
   (end_list)
   (repeat (length AddLst)
    (set_tile "laylst" (itoa LstCnt))
    (setq SelLst (cons LstCnt SelLst)
          LstCnt (1+ LstCnt)
    )
   )
   (if AddLst
    (progn
     (set_tile "error" "Selected item(s) have no DrawOrder!")
     (setq SelLst (MeList2String (reverse SelLst) " ")
           Me:Fst T
     )
    )
   )
   (action_tile "laylst" "(setq SelLst (MeMoveControl $value))")
   (action_tile "lay_up" "(setq	SelLst (MeMoveListItems SelLst T))")
   (action_tile "lay_dn" "(setq	SelLst (MeMoveListItems SelLst nil))")
   (action_tile "cancel" "(setq Me:Lst nil)")
   (action_tile "accept" "(setq Me:Pos (done_dialog 0))")
   (start_dialog)
   (if Me:Lst
    (progn
     (setq OldCmd (getvar "CMDECHO"))
     (setvar "CMDECHO" 0)
     (princ "\n")
     (foreach memb (reverse Me:Lst)
      (if (setq CurSet (ssget "_X" (list (cons 8 memb))))
       (progn
        (princ "\rProcessing objects... ")
        (MeDoSpin)
        (vl-cmdf "_.DRAWORDER" CurSet "")
        (if (= (getvar "CMDACTIVE") 1) (vl-cmdf "_FRO"))
       )
      )
     )
     (setvar "CMDECHO" OldCmd)
     (setq DefLst (mapcar '(lambda (l) (cons 1 l)) Me:Lst))
     (MeSetDictRecs "DrawOrderByLayer" "DefaultLayerList" DefLst)
    )
   )
   (vla-EndUndoMark Me:AcD)
  )
 )
 (princ)
)
;
; == Subs =====================================================================
;
; == Function MeAll2String
; Converts all variable types to a string.
; Arguments [Type]:
;   Val = Value to convert [INT/REAL/LIST/STR]
; Return [Type]:
;   > Converted value [STR]
; Notes:
;   - None
;
(defun MeAll2String (Val)
 (cond
  ((= (type Val) 'INT ) (itoa Val))
  ((= (type Val) 'REAL) (rtos Val))
  ((= (type Val) 'LIST) (MeList2String Val " "))
  ((= (type Val) 'STR ) Val)
  (T "")
 )
)
;
; == Function MeDoSpin
; Writes a spinner to the command line.
; Arguments [Type]:
;   --- = 
; Return [Type]:
;   > Null
; Notes:
;   - Requires the global spinner variable Me:Prp
;
(defun MeDoSpin ()
 (setq Me:Prp (cond
               ((eq Me:Prp "|")  "/")
               ((eq Me:Prp "/")  "-")
               ((eq Me:Prp "-") "\\")
               ((eq Me:Prp "\\") "|")
               (T                "|")
              )
 )
 (princ Me:Prp)
 (princ)
)
;
; == Function MeEditList
; Delete, replace or append a list.
; Arguments [Type]:
;   Mde = Mode  1 append  [INT]
;         Mode  0 replace [INT]
;         Mode -1 delete  [INT]
;   Pos = Position in list [INT]
;   Val = New value [INT/REAL/STR/LIST]
;   Lst = List [LIST]
; Return [Type]:
;   > Modified list [LIST]
; Notes:
;   - Delete and replace require the position (Pos) in list.
;   - Append and replace require the new value (Val).
;   - :vlax-null items are not allowed in the list argument
;
(defun MeEditList (Mde Pos Val Lst / Countr TmpLst TmpVal)
 (if (= Mde 1)
  (reverse (cons Val (reverse Lst)))
  (progn 
   (setq Countr -1
         TmpVal (if (= Mde -1) :vlax-null Val)
         TmpLst (mapcar
                '(lambda (l)
                  (if (= Pos (setq Countr (1+ Countr))) TmpVal l)
                 ) Lst
                )
   )
   (vl-remove :vlax-null TmpLst)
  )
 )
)
;
; == Function MeGetDictRecs
; Reads data from a dictionary by name and key.
; Arguments [Type]:
;   Dnm = Dictionary name [STR]
;   Key = Key name [STR]
; Return [Type]:
;   > Data list '((1 . "ABC) (70 . 33)...) [LIST]
;   > False if dictionary and/or key not found
; Notes:
;   - Requires the global ActiveDocument variable Me:AcD
;
(defun MeGetDictRecs (Dnm Key / DicObj XrcDat XrcObj XrcTyp)
 (if (and
      (not
       (vl-catch-all-error-p
        (setq DicObj (vl-catch-all-apply
                     'vla-Item
                      (list (vla-get-Dictionaries Me:AcD) Dnm)
                     )
        )
       )
      )
      (not
       (vl-catch-all-error-p
        (setq XrcObj (vl-catch-all-apply 'vla-Item (list DicObj Key)))
       )
      )
     )
  (progn
   (vlax-invoke-method XrcObj 'GetXRecordData 'XrcTyp 'XrcDat)
   (mapcar 'cons
    (vlax-SafeArray->List XrcTyp)
    (mapcar 'vlax-variant-value (vlax-SafeArray->List XrcDat))
   )
  )
 )
)
;
; == Function MeGetLayerNames
; Returns a list of all layer names
; Arguments [Typ]:
;   --- =
; Return [Typ]:
;   > Layer names [LIST]
; Notes:
;   - Requires the global ActiveDocument variable Me:AcD
; 
(defun MeGetLayerNames ( / NmeLst)
 (vlax-for Obj (vla-get-Layers Me:AcD)
  (setq NmeLst (cons (vla-get-name Obj) NmeLst))
 )
 (reverse NmeLst)
)
;
; == Function MeGetMassoc
; Get multiple associative values from a list.
; Arguments [Type]:
;   Key = Key to search [INT]
;   Lst = Dotted pair list [LIST]
; Return [Type]:
;   > List of values [LIST]
; Notes:
;   - Credits to Michael Puckett
;
(defun MeGetMassoc (Key Lst / RetVal TmpLst TmpVal)
 (setq TmpLst Lst)
 (reverse
  (progn
   (while (setq TmpVal (assoc Key TmpLst))
    (setq RetVal (cons (cdr TmpVal) RetVal)
          TmpLst (cdr (member TmpVal TmpLst))
    )
   )
   RetVal
  )
 )
)
;
; == Function MeList2Array
; Converts a list into an array.
; Arguments [Type]:
;   Typ = Datatype [INT]
;         Constants:
;         - vlax-vbBoolean
;         - vlax-vbDouble
;         - vlax-vbInteger
;         - vlax-vbLong
;         - vlax-vbObject
;         - vlax-vbSingle
;         - vlax-vbString
;         - vlax-vbVariant
;   Lst = Standard list [LIST]
; Return [Type]:
;   > Array [VARIANT]
; Notes:
;   - Can't be used for dotted pair or nested lists.
;
(defun MeList2Array (Typ Lst)
 (vlax-make-variant
  (vlax-safearray-fill
   (vlax-make-safearray Typ (cons 0 (1- (length Lst))))
   Lst
  )
 )
)
;
; == Function MeList2String
; Converts a list to a string.
; Arguments [Type]:
;   Lst = List [LIST]
;   Del = Delimiter pattern [STR]
; Return [Type]:
;   > Converted list [STR]
; Notes:
;   - None
;
(defun MeList2String (Lst Del)
 (apply 'strcat
  (cons
   (MeAll2String (car Lst))
   (mapcar
   '(lambda (l) (strcat Del (MeAll2String l)))
    (cdr Lst)
   )
  )
 )
)
;
; == Function MeLoadDcl
; Load function for DCL-Files.
; Arguments [Type]:
;   Fnm = File name [STR]
; Return [Type]:
;   > DCL-ID [INT]
;   > nil if an error occurs
; Notes:
;   - Uses the global variable Me:Dcl
;
(defun MeLoadDcl (Fnm / Dcl_Id)
 (cond
  ((cdr (assoc Fnm Me:Dcl)))
  ((not (findfile (strcat Fnm ".dcl")))
   (alert (strcat "Application file '" Fnm ".dcl' not found. "))
  )
  ((< (setq Dcl_Id (load_dialog Fnm)) 0)
   (alert (strcat "Error on loading application file '" Fnm ".dcl'. "))
  )
  (T
   (setq Me:Dcl (cons (cons Fnm Dcl_Id) Me:Dcl))
   Dcl_Id
  )
 )
)
;
; == Function MeMoveControl
; Controls the up/down buttons.
; Arguments [Type]:
;   Sel = Selected item(s) [STR]
; Return [Type]:
;   >  Selected item(s) [STR]
; Notes:
;   - Requires the global list item variable Me:Lst
;   - Requires the global first called variable Me:Fst
;
(defun MeMoveControl (Sel / TmpLst)
 (if Me:Fst
  (progn
   (set_tile "error" "")
   (setq Me:Fst nil)
  )
 )
 (setq TmpLst (mapcar 'atoi (MeString2List Sel " ")))
 (if (= (apply 'min TmpLst) 0)
  (mode_tile "lay_up" 1)
  (mode_tile "lay_up" 0)
 )
 (if (= (apply 'max TmpLst) (1- (length Me:Lst)))
  (mode_tile "lay_dn" 1)
  (mode_tile "lay_dn" 0)
 )
 Sel
)
;
; == Function MeMoveListItems
; Moves the selected list items one step up or down in list.
; Arguments [Type]:
;   Sel = Selected item(s) [STR]
;   Dir = Move direction [BOOLEAN]
;         T   = up
;         nil = down
; Return [Type]:
;   > Moved selected item(s) [STR]
; Notes:
;   - Requires the global list item variable Me:Lst
;
(defun MeMoveListItems (Sel Dir / CurItm NxtItm SelLst TmpLst)
 (setq TmpLst (mapcar 'atoi (MeString2List Sel " "))
       SelLst (mapcar 'itoa (mapcar (if Dir '1- '1+) TmpLst))
 )
 (start_list "laylst")
 (mapcar 'add_list Me:Lst)
 (end_list)
 (mapcar '(lambda (l) (set_tile "laylst" l)) SelLst)
 (foreach memb (if Dir TmpLst (reverse TmpLst))
  (setq CurItm (nth memb Me:Lst)
        NxtItm (nth ((if Dir 1- 1+) memb) Me:Lst)
  )
  (setq Me:Lst (MeEditList 0 memb NxtItm Me:Lst)
        Me:Lst (MeEditList 0 ((if Dir 1- 1+) memb) CurItm Me:Lst)
  )
 )
 (start_list "laylst")
 (mapcar 'add_list Me:Lst)
 (end_list)
 (mapcar '(lambda (l) (set_tile "laylst" l)) SelLst)
 (MeMoveControl (MeList2String SelLst " "))
)
;
; == Function MeOpenDcl
; Opens a DCL dialog.
; Arguments [Type]:
;   Nme = DCL name [STR]
;   Pos = Last screen position [LIST]
;   Did = DCL-ID [INT]
; Return [Type]:
;   > Null
; Notes:
;   - Requires the global application variable MeCpr
;
(defun MeOpenDcl (Nme Pos Did / DlgFlg)
 (setq DlgFlg (if Pos
               (new_dialog Nme Did "" Pos)
               (new_dialog Nme Did)
              )
 )
 ; Copyright - Note (May be never deleted)
 (if DlgFlg (set_tile "cpyrgt" "2006 MENZI ENGINEERING GmbH, Switzerland"))
 (princ)
)
;
; == Function MeSetDictRecs
; Creates a dictionary with key by the given arguments and writes data.
; Arguments [Type]:
;   Dnm = Dictionary name [STR]
;   Key = Key name [STR]
;   Dat = Data list '((1 . "ABC) (70 . 33)...) [LIST]
; Return [Type]:
;   > Null
; Notes:
;   - If key already exists, data will be overwritten.
;   - Requires the global ActiveDocument variable Me:AcD
;
(defun MeSetDictRecs (Dnm Key Dat / DicObj DicCol XrcDat XrcObj XrcTyp)
 (setq DicCol (vla-get-Dictionaries Me:AcD))
 (if (vl-catch-all-error-p
      (setq DicObj (vl-catch-all-apply 'vla-Item (list DicCol Dnm)))
     )
  (setq DicObj (vla-add DicCol Dnm))
 )
 (if (vl-catch-all-error-p
      (setq XrcObj (vl-catch-all-apply 'vla-Item (list DicObj Key)))
     )
  (setq XrcObj (vla-AddXRecord DicObj Key))
 )
 (setq XrcTyp (MeList2Array vlax-vbInteger (mapcar 'car Dat))
       XrcDat (MeList2Array vlax-vbVariant (mapcar 'cdr Dat))
 )
 (vla-SetXRecordData XrcObj XrcTyp XrcDat)
 (princ)
)
;
; == Function MeString2List
; Converts a string to a list.
; Arguments [Type]:
;   Stg = String [STR]
;   Del = Delimiter pattern [STR]
; Return [Type]:
;   > Converted string [LIST]
;   > False if string empty
; Notes:
;   - None
;
(defun MeString2List (Stg Del / DelLgt StrPos TmpLst TmpStr)
 (if (not (eq Stg ""))
  (progn
   (setq TmpStr Stg
         DelLgt (1+ (strlen Del))
   )
   (while (setq StrPos (vl-string-search Del TmpStr))
    (setq TmpLst (cons (substr TmpStr 1 StrPos) TmpLst)
          TmpStr (substr TmpStr (+ StrPos DelLgt))
    )
   )
   (setq TmpLst (cons TmpStr TmpLst))
   (reverse TmpLst)
  )
 )
)
;
; == Copyright - Note (May be never deleted) ==================================
;
(princ "\n---------------------------------------------")
(princ "\n 2006 MENZI ENGINEERING GmbH, Switzerland ")
(princ "\n---------------------------------------------")
(princ
 (strcat
  "\nType DrawOrderByLayer "
  "into the command line to start the program..."
 )
)
(princ)
;
; == End DrawOrderByLayer =====================================================
